/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.processor.vorgang;

import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.stereotype.Service;

import de.ozgcloud.apilib.vorgang.OzgCloudVorgangId;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangService;
import de.ozgcloud.processor.ProcessorManagerConfiguration;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;

@ConditionalOnProperty(prefix = "ozgcloud.processors[0]", name = "address")
@Service(ProcessorManagerConfiguration.VORGANG_SERVICE_NAME) // NOSONAR
@RequiredArgsConstructor
public class VorgangService {

	@Qualifier(ProcessorManagerConfiguration.OZGCLOUD_VORGANG_SERVICE_NAME) // NOSONAR
	private final OzgCloudVorgangService ozgCloudService;

	@Qualifier(ProcessorManagerConfiguration.VORGANG_MAPPER_NAME) // NOSONAR
	private final ProcessorVorgangMapper mapper;

	public Vorgang getVorgang(@NonNull VorgangId id) {
		var loaded = ozgCloudService.getById(OzgCloudVorgangId.from(id.toString()));

		return mapper.mapVorgang(loaded);
	}
}
