/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.processor.processor;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Optional;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.Spy;
import org.springframework.http.client.reactive.ClientHttpConnector;
import org.springframework.web.reactive.function.client.WebClient;

class ProcessorConfigurationTest {

	@Spy
	@InjectMocks
	private ProcessorConfiguration processorConfiguration;

	@Mock
	private WebClient.Builder webClienBuilder;
	@Mock
	private ClientHttpConnector clientHttpConnector;

	@Spy
	private Optional<ProxyProperties> proxyProperties = Optional.of(ProxyPropertiesTestFactory.create());

	@Nested
	class TestCreateProxy {

		@Test
		void shouldCallCreateProxyConnector() {
			processorConfiguration.webClient(webClienBuilder);

			verify(processorConfiguration).createProxyConnector(proxyProperties.get());
		}

		@Test
		void shouldAddProxyConnector() {
			doReturn(clientHttpConnector).when(processorConfiguration).createProxyConnector(any());

			processorConfiguration.webClient(webClienBuilder);

			verify(webClienBuilder).clientConnector(clientHttpConnector);
		}
	}

	@Test
	void shouldNotAddProxyConnector() {
		processorConfiguration = Mockito.spy(new ProcessorConfiguration(Optional.empty()));

		processorConfiguration.webClient(webClienBuilder);

		verify(processorConfiguration, never()).createProxyConnector(any());
		verify(webClienBuilder, never()).clientConnector(any());
	}
}