/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.processor.processor;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.context.ApplicationEventPublisher;

import de.ozgcloud.apilib.vorgang.OzgCloudVorgangTestFactory;
import de.ozgcloud.command.CommandCreatedEvent;
import de.ozgcloud.command.CommandTestFactory;
import de.ozgcloud.command.VorgangCreatedEvent;
import de.ozgcloud.common.test.ITCase;
import de.ozgcloud.processor.result.ResultService;
import de.ozgcloud.processor.vorgang.VorgangId;
import de.ozgcloud.processor.vorgang.VorgangService;
import de.ozgcloud.processor.vorgang.VorgangTestFactory;

@ITCase
class ProcessorEventListenerITCase {

	@MockBean
	private ResultService resultService;
	@MockBean
	private VorgangService service;
	@MockBean
	private ProcessorService processorService;

	@Autowired
	private ApplicationEventPublisher publisher;

	@Test
	void shouldLoadVorgang() {
		when(service.getVorgang(any())).thenReturn(VorgangTestFactory.create());
		var event = new VorgangCreatedEvent(OzgCloudVorgangTestFactory.ID.toString());

		publisher.publishEvent(event);

		verify(service).getVorgang(VorgangId.from(OzgCloudVorgangTestFactory.ID.toString()));
	}

	@DisplayName("On command listener")
	@Nested
	class TestOnCommandListener {

		@Test
		void shouldTriggerForExecuteProcessor() {
			var command = CommandTestFactory.createBuilder().order(ProcessorEventListener.PROCESS_VORGANG_ORDER).build();

			publisher.publishEvent(new CommandCreatedEvent(command));

			verify(service).getVorgang(VorgangId.from(CommandTestFactory.VORGANG_ID));
		}
	}
}
