/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.processor.result;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Spy;

import de.ozgcloud.processor.processor.ProcessorResult;
import de.ozgcloud.processor.processor.ProcessorResultTestFactory;
import de.ozgcloud.processor.vorgang.VorgangTestFactory;

class AktennotizFactoryTest {

	@Spy
	private AktennotizFactory factory;

	private ProcessorResult processorResult = ProcessorResultTestFactory.create();

	@Nested
	class TestCreateFromProcessorResult {

		@Test
		void shouldSetVorgangId() {
			var aktennotiz = factory.createFrom(processorResult);

			assertThat(aktennotiz.getVorgangId()).isEqualTo(processorResult.getVorgangId().toString());
		}

		@Test
		void shouldSetSeverity() {
			var aktennotiz = factory.createFrom(processorResult);

			assertThat(aktennotiz.getSeverity()).isEqualTo(processorResult.getBody().getSeverity());
		}

		@Test
		void shouldSetHeadline() {
			var aktennotiz = factory.createFrom(processorResult);

			assertThat(aktennotiz.getHeadline()).isEqualTo(processorResult.getBody().getHeadline());
		}

		@Test
		void shouldSetText() {
			var aktennotiz = factory.createFrom(processorResult);

			assertThat(aktennotiz.getText()).isEqualTo(processorResult.getBody().getText());
		}
	}

	@Nested
	class TestCreateErrorAktennotiz {

		private static final String VORGANG_ID = VorgangTestFactory.ID.toString();

		private static final String PROCESSOR_NAME = "processorName";

		private static final String EXCEPTION_ID = "exceptionId";

		@Test
		void shouldSetVorgangId() {
			var aktennotiz = createErrorAktennotiz();

			assertThat(aktennotiz.getVorgangId()).isEqualTo(processorResult.getVorgangId().toString());
		}

		@Test
		void shouldSetSeverity() {
			var aktennotiz = createErrorAktennotiz();

			assertThat(aktennotiz.getSeverity()).isEqualTo(Severity.ERROR);
		}

		@Test
		void shouldSetHeadline() {
			var aktennotiz = createErrorAktennotiz();

			assertThat(aktennotiz.getHeadline()).isEqualTo("Fehler");
		}

		@Test
		void shouldSetText() {
			var errorText = "errorText";
			doReturn(errorText).when(factory).buildErrorText(anyString(), anyString());

			var aktennotiz = createErrorAktennotiz();

			assertThat(aktennotiz.getText()).isEqualTo(errorText);
		}

		private Aktennotiz createErrorAktennotiz() {
			return factory.createErrorAktennotiz(VORGANG_ID, PROCESSOR_NAME, EXCEPTION_ID);
		}
	}

	@Nested
	class TestBuildErrorText {

		@Test
		void shouldAddProcessorName() {
			var processorName = "processorName";

			var errorText = factory.buildErrorText(processorName, "");

			assertThat(errorText).contains(processorName);
		}

		@Test
		void shouldAddExceptionId() {
			var exceptionId = "exceptionId";

			var errorText = factory.buildErrorText("", exceptionId);

			assertThat(errorText).contains(exceptionId);
		}
	}
}